/**
  *  \file client/screens.h
  *
  *  (c) 2001-2020 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_SCREENS_H
#define PCC_V2_CLIENT_SCREENS_H

#include "client/wtile.h"
#include "cpluslib/string.h"
#include "game/cargo.h"
#include "game/coord.h"
#include "game/objtypes.h"
#include "io/io.h"
#include "game/playerset.h"

class GObject;
class GOutbox;
class IntUserInterfaceBinding;

/* client/screens.cc */

/** Screen Identifier. Identifies a screen the user may be in, i.e. a major
    position in the main state machine. */
class WScreenId {
 public:
    enum ScreenId {
        NullScreen        = -1,
        RaceScreen        = 0,

        ShipScreen        = 1,
        PlanetScreen      = 2,
        BaseScreen        = 3,
        Starchart         = 4,

        HistoryScreen     = 6,

        FleetScreen       = 10,
        ShipTaskScreen    = 11,
        PlanetTaskScreen  = 12,
        BaseTaskScreen    = 13
    };

    WScreenId();
    WScreenId(ScreenId screen_id, int id);
    WScreenId(GPoint pt);

    ScreenId getScreenId() const;
    int      getId() const;
    GPoint   getPos() const;
    bool     isNonNull() const;
    bool     isValid() const;

    string_t toString();

    bool operator==(const WScreenId& other) const;
    bool operator!=(const WScreenId& other) const;

 private:
    ScreenId screen_id;
    int x, y;
};


inline WScreenId::ScreenId
WScreenId::getScreenId() const
{
    return screen_id;
}

inline bool
WScreenId::isNonNull() const
{
    return screen_id != NullScreen;
}

inline bool
WScreenId::operator==(const WScreenId& other) const
{
    return screen_id == other.screen_id
        && x == other.x
        && y == other.y;
}

inline bool
WScreenId::operator!=(const WScreenId& other) const
{
    return !operator==(other);
}

/* client/scr-msg.cc */
void doSendOrReviseMessage();
void doSendMessage(string_t text, GPlayerSet receivers);
void doEditMessage(GOutbox& outbox, int index);

/* client/scr-env.cc */
void doPlanetScan(int pid);

/*********************** User Interface Interaction **********************/

bool postGoToScreen(IntUserInterfaceBinding& ui, int screen, int id);
bool postGoToChart(IntUserInterfaceBinding& ui, GPoint pt, const GObject* origin);
bool postGoToObject(IntUserInterfaceBinding& ui, GObject& obj, bool preferBases);

#endif
