/**
  *  \file int/if/objcontext.cc
  *  \brief Interpreter Game Interface: Object Context
  *
  *  (c) 2001-2020 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/if/objcontext.h"
#include "game/objtypes.h"

IntObjectContext::~IntObjectContext()
{ }

/** Construction utility for "Make" function.
    This determines whether there is an object of this object type, and,
    if so, enters its context. Otherwise, deletes this object.

    Call this as "return (new IntObjectContext(xx, 0))->initFirst())"
    from your "Make" function. */
IntContext*
IntObjectContext::initFirst()
{
    index = type.findNextIndex(0);
    if (index != 0) {
        // Found
        return this;
    } else {
        // Not found
        delete this;
        return 0;
    }
}

// Implementation of IntContext methods:
bool
IntObjectContext::next()
{
    if (int n = type.findNextIndexNoWrap(index, false)) {
        index = n;
        return true;
    } else {
        return false;
    }
}

GObject*
IntObjectContext::getObject()
{
    if (type.isValidIndex(index)) {
        return &type.getObjectByIndex(index);
    } else {
        return 0;
    }
}

GUniverse*
IntObjectContext::getCurrentUniverse() const
{
    if (type.isValidIndex(index)) {
        return type.getUniverseByIndex(index);
    } else {
        return 0;
    }
}
