/**
  *  \file u/t_int_data.cc
  *
  *  (c) 2001-2020 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/data.h"
#include "u/t_int.h"
#include "io/memstream.h"

void
IntDataTestSuite::testLoad()
{
    static const char data[] = {
        0, 0, 0, 0, 0, 0,        // real 0.0
        0, 2, 5, 4, 0, 0,        // int 1029
        0, 1, 0, 0, 0, 0,        // null
        0, 4, 0, 0, 0, 0,        // blank string
        0, 3, 1, 0, 0, 0,        // bool true
        0, 6, 7, 0, 0, 0,        // long string, 7 chars
        0, 4, 0, 0, 1, 0,        // string, not empty,
        char(0x83), 0, 0, 0, 0, 0x20,  // real 5.0
        0, 5, 0, 0, 0x40, 0x40,  // float 3.0
        // = 9 entries, 54 bytes

        'a', 'b', 'c', 'd', 'e', 'f', 'g',
        3, 'X', 'Y', 'Z',
        // +11 bytes, 65 bytes total

        '1', '2', '3'
    };
    IntStringValue* sv = 0;
    IntFloatValue* fv  = 0;
    IntIntValue* iv    = 0;

    MemoryStream mem(string_t(data, sizeof(data)));
    IntDataSegment seg;
    seg.load(mem, 9);

    // Make sure 65 bytes consumed
    TS_ASSERT_EQUALS(mem.getPos(), 65);

    // First entry
    TS_ASSERT(seg[0]);
    TS_ASSERT((fv = dynamic_cast<IntFloatValue*>(seg[0])) != 0);
    TS_ASSERT_EQUALS(fv->getValue(), 0.0);

    // Second entry
    TS_ASSERT(seg[1]);
    TS_ASSERT((iv = dynamic_cast<IntIntValue*>(seg[1])) != 0);
    TS_ASSERT_EQUALS(iv->getValue(), 1029);
    TS_ASSERT(!iv->isBool());

    // Third entry
    TS_ASSERT(!seg[2]);

    // Fourth entry
    TS_ASSERT(seg[3]);
    TS_ASSERT((sv = dynamic_cast<IntStringValue*>(seg[3])) != 0);
    TS_ASSERT_EQUALS(sv->getValue(), "");

    // Fifth entry
    TS_ASSERT(seg[4]);
    TS_ASSERT((iv = dynamic_cast<IntIntValue*>(seg[4])) != 0);
    TS_ASSERT_EQUALS(iv->getValue(), 1);
    TS_ASSERT(iv->isBool());

    // Sixth entry
    TS_ASSERT(seg[5]);
    TS_ASSERT((sv = dynamic_cast<IntStringValue*>(seg[5])) != 0);
    TS_ASSERT_EQUALS(sv->getValue(), "abcdefg");

    // Seventh entry
    TS_ASSERT(seg[6]);
    TS_ASSERT((sv = dynamic_cast<IntStringValue*>(seg[6])) != 0);
    TS_ASSERT_EQUALS(sv->getValue(), "XYZ");

    // Eighth entry
    TS_ASSERT(seg[7]);
    TS_ASSERT((fv = dynamic_cast<IntFloatValue*>(seg[7])) != 0);
    TS_ASSERT_EQUALS(fv->getValue(), 5.0);

    // Ninth entry
    TS_ASSERT(seg[8]);
    TS_ASSERT((fv = dynamic_cast<IntFloatValue*>(seg[8])) != 0);
    TS_ASSERT_EQUALS(fv->getValue(), 3.0);

    // Tenth and following entries (not deserialized)
    TS_ASSERT(!seg[9]);
    TS_ASSERT(!seg[10]);
    TS_ASSERT(!seg[11]);
}
