/**
  *  \file u/t_int_notify.cc
  *
  *  (c) 2001-2020 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/notify.h"

#include "u/t_int.h"

void
IntNotifyTestSuite::testNotify()
{
    // create empty store
    IntNotificationMessageStore store;
    TS_ASSERT_EQUALS(store.getNumMessages(), 0U);

    // add a message
    IntNotificationMessage* msg = store.addNewMessage("foo", "bar");
    TS_ASSERT(msg != 0);
    TS_ASSERT_EQUALS(store.getNumMessages(), 1U);
    TS_ASSERT_EQUALS(store.getIndexOfMessage(msg), 0U);
    TS_ASSERT_EQUALS(store.getMessageByIndex(0), msg);
    TS_ASSERT_EQUALS(msg->getHeader(), "foo");
    TS_ASSERT_EQUALS(msg->getBody(), "bar");

    // add another message
    IntNotificationMessage* msg2 = store.addNewMessage("foo2", "bar2");
    TS_ASSERT(msg2 != 0);
    TS_ASSERT(msg2 != msg);
    TS_ASSERT_EQUALS(store.getNumMessages(), 2U);
    TS_ASSERT_EQUALS(store.getIndexOfMessage(msg2), 1U);
    TS_ASSERT_EQUALS(store.getMessageByIndex(1), msg2);
    TS_ASSERT_EQUALS(msg2->getHeader(), "foo2");
    TS_ASSERT_EQUALS(msg2->getBody(), "bar2");

    // delete one
    store.removeMessageByIndex(0);
    TS_ASSERT_EQUALS(store.getNumMessages(), 1U);
    TS_ASSERT_EQUALS(store.getIndexOfMessage(msg2), 0U);
    TS_ASSERT_EQUALS(store.getMessageByIndex(0), msg2);

    // function will be left with one message in the store,
    // test that this works.
}
